#!/usr/bin/python
import numpy as np
import cmath
from cmath import*
from sympy.physics.quantum.dagger import Dagger
import math
from lmfit import Model
import time
#import matplotlib.pyplot as plt

a = 9.83999995859962e-10
aGr = 2.46e-10
hbar = 6.582119e-16
pi = math.pi
tau = 1
Ry = 13.605693
EF = 0.0
ED = 0.00
gamma0 = 2.55517409
gamma1 = 0.41167674
gamma3 = -0.31096519
gamma4 = -0.17848374
V = -0.0005
delta = 0.01337065
LIA1 = 0.00112134
LIB1 = -0.00119197
LIA2 = 12e-6
LIB2 = 12e-6
L0 = 0
LBR1 = -5.1604e-04
LBR2 = 0
L1 = 0
L3 = 0
L4 = 0
DL4 = 0
# In order to add exchange coupling to one layer, we introduce on the diagonal terms for A1 and B1
BexA1 = 0.0
BexB1 = 0.0
BexA2 = 0.0
BexB2 = 0.0

#DFT daten#
data = np.loadtxt('../0Vnm/bandsK/DFTbandsK.dat')
kx = []
ky = []

k1 = (data[:,0]-1./3.).real 	#shift by (1/3, 1/3) due to the K point
k2 = (data[:,1]-1./3.).real
kmax=len(k1)

for i in range(0,kmax):
        kx.append(((4*pi/(sqrt(3)*a))*(k1[i]*sqrt(3.)/2.)).real)
        ky.append(((4*pi/(sqrt(3)*a))*(k1[i]*(1./2.)+k2[i])).real)

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def Hamiltonian(kx,ky,tau,ED,gamma0,gamma1,gamma3,gamma4,V,delta,LIA1,LIA2,LIB1,LIB2,L0,LBR1,LBR2,L1,L3,L4,DL4,BexA1,BexB1,BexA2,BexB2):

	#FK = exp(  complex(0.0,a*ky/  sqrt(3.0)  )  )* (  1+2*exp(  complex(  0.0,-sqrt(3.0)*a*ky/2.0  )  )*cos(a*kx/2.0)  )	#structure function, non linearized, momentum must be at K, do not shift
	#CFK = np.conjugate(FK)

	FK = (1./2.)*sqrt(3.)*(aGr)*complex(-tau*kx,ky)	#linearized version of structure function, expanded around K point
	CFK = np.conjugate(FK)
	
	h11 = delta+tau*LIA1+V-BexA1
	h12 = 0.0
	h13 = FK*gamma0
	h14 = (1./2.)*complex(0.0,(L0+2*LBR1)*(1-tau))
	h15 = gamma4*CFK
	h16 = (1./2.)*complex(0.0,(DL4+L4)*(1+tau))
	h17 = gamma1+L1*tau
	h18 = 0.0

	h21 = 0.0
	h22 = delta-LIA1*tau+V+BexA1
	h23 = (1./2.)*complex(0.0,(L0+2*LBR1)*(1+tau))
	h24 = FK*gamma0
	h25 = (1./2.)*complex(0.0,(DL4+L4)*(1-tau))
	h26 = gamma4*CFK
	h27 = 0.0
	h28 = -gamma1-L1*tau

	h31 = CFK*gamma0
	h32 = -(1./2.)*complex(0.0,(L0+2*LBR1)*(1+tau))
	h33 = -LIB1*tau+V-BexB1
	h34 = 0.0
	h35 = FK*gamma3
	h36 = (1./2.)*complex(0.0,L3*(1-tau))
	h37 = CFK*gamma4
	h38 = -(1./2.)*complex(0.0,(-DL4+L4)*(1+tau))

	h41 = -(1./2.)*complex(0.0,(L0+2*LBR1)*(1-tau))
	h42 = CFK*gamma0
	h43 = 0.0
	h44 = LIB1*tau+V+BexB1
	h45 = (1./2.)*complex(0.0,L3*(1+tau))
	h46 = FK*gamma3
	h47 = -(1./2.)*complex(0.0,(-DL4+L4)*(1-tau))
	h48 = CFK*gamma4

	h51 = FK*gamma4
	h52 =  -(1./2.)*complex(0.0,(DL4+L4)*(1-tau))
	h53 = CFK*gamma3
	h54 = -(1./2.)*complex(0.0,L3*(1+tau))
	h55 = LIA2*tau-V-BexA2
	h56 = 0.0
	h57 = FK*gamma0
	h58 = -(1./2.)*complex(0.0,(L0-2*LBR2)*(1-tau))

	h61 = -(1./2.)*complex(0.0,(DL4+L4)*(1+tau))
	h62 = FK*gamma4
	h63 = -(1./2.)*complex(0.0,L3*(1-tau))
	h64 = CFK*gamma3
	h65 = 0.0
	h66 = -LIA2*tau-V+BexA2
	h67 = -(1./2.)*complex(0.0,(L0-2*LBR2)*(1+tau))
	h68 = FK*gamma0

	h71 = gamma1+L1*tau
	h72 = 0.0
	h73 = FK*gamma4
	h74 = (1./2.)*complex(0.0,(-DL4+L4)*(1-tau))
	h75 = CFK*gamma0
	h76 = (1./2.)*complex(0.0,(L0-2*LBR2)*(1+tau))
	h77 = delta-LIB2*tau-V-BexB2
	h78 = 0.0

	h81 = 0.0
	h82 = gamma1-L1*tau
	h83 = (1./2.)*complex(0.0,(-DL4+L4)*(1+tau))
	h84 = FK*gamma4
	h85 = (1./2.)*complex(0.0,(L0-2*LBR2)*(1-tau))
	h86 = CFK*gamma0
	h87 = 0.0
	h88 = delta+LIB2*tau-V+BexB2
	

	H = np.array([[h11,h12,h13,h14,h15,h16,h17,h18],
		      [h21,h22,h23,h24,h25,h26,h27,h28],
		      [h31,h32,h33,h34,h35,h36,h37,h38],
		      [h41,h42,h43,h44,h45,h46,h47,h48],
		      [h51,h52,h53,h54,h55,h56,h57,h58], 
		      [h61,h62,h63,h64,h65,h66,h67,h68],
		      [h71,h72,h73,h74,h75,h76,h77,h78],
		      [h81,h82,h83,h84,h85,h86,h87,h88]])

	Sx = np.array([ [0,1,0,0,0,0,0,0],
			[1,0,0,0,0,0,0,0],
			[0,0,0,1,0,0,0,0],
			[0,0,1,0,0,0,0,0],
			[0,0,0,0,0,1,0,0],
			[0,0,0,0,1,0,0,0],
			[0,0,0,0,0,0,0,1],
			[0,0,0,0,0,0,1,0]])

	Sy = np.array([ [0,complex(0,-1),0,0,0,0,0,0],
			[complex(0,1),0,0,0,0,0,0,0],
			[0,0,0,complex(0,-1),0,0,0,0],
			[0,0,complex(0,1),0,0,0,0,0],
			[0,0,0,0,0,complex(0,-1),0,0],
			[0,0,0,0,complex(0,1),0,0,0],
			[0,0,0,0,0,0,0,complex(0,-1)],
			[0,0,0,0,0,0,complex(0,1),0]])

	Sz = np.array([ [1,0,0,0,0,0,0,0],
			[0,-1,0,0,0,0,0,0],
			[0,0,1,0,0,0,0,0],
			[0,0,0,-1,0,0,0,0],
			[0,0,0,0,1,0,0,0],
			[0,0,0,0,0,-1,0,0],
			[0,0,0,0,0,0,1,0],
			[0,0,0,0,0,0,0,-1]])

	evals, evecs = np.linalg.eigh(H)

	ew=[[],[],[],[],[],[],[],[]]
	spinx=[[],[],[],[],[],[],[],[]]
	spiny=[[],[],[],[],[],[],[],[]]
	spinz=[[],[],[],[],[],[],[],[]]



	for i in range(0,8):
		ew[i] = evals[i]+ED
		EV = evecs[:,i]
		DEV = Dagger(evecs[:,i])
		spinx[i]=1./2.*(DEV.dot(Sx.dot(EV))).real
		spiny[i]=1./2.*(DEV.dot(Sy.dot(EV))).real	
		spinz[i]=1./2.*(DEV.dot(Sz.dot(EV))).real	
	
	
	return ew[0],ew[1],ew[2],ew[3],ew[4],ew[5],ew[6],ew[7],spinx[0],spinx[1],spinx[2],spinx[3],spinx[4],spinx[5],spinx[6],spinx[7],spiny[0],spiny[1],spiny[2],spiny[3],spiny[4],spiny[5],spiny[6],spiny[7],spinz[0],spinz[1],spinz[2],spinz[3],spinz[4],spinz[5],spinz[6],spinz[7]
	
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

for i in range(0,kmax):
	EDFT = (data[i][2:10]-EF)
	SXDFT = data[i][10:18]
	SYDFT = data[i][18:26]
	SZDFT = data[i][26:34]
	EW = Hamiltonian(kx[i],ky[i],tau,ED,gamma0,gamma1,gamma3,gamma4,V,delta,LIA1,LIA2,LIB1,LIB2,L0,LBR1,LBR2,L1,L3,L4,DL4,BexA1,BexB1,BexA2,BexB2)
	if kx[i] > 0:
		print (-(sqrt(kx[i]**2+ky[i]**2)*1e-10).real,EW[0],EW[1],EW[2],EW[3],EW[4],EW[5],EW[6],EW[7],EW[8],EW[9],EW[10],EW[11],EW[12],EW[13],EW[14],EW[15],EW[16],EW[17],EW[18],EW[19],EW[20],EW[21],EW[22],EW[23],EW[24],EW[25],EW[26],EW[27],EW[28],EW[29],EW[30],EW[31],EDFT[0],EDFT[1],EDFT[2],EDFT[3],EDFT[4],EDFT[5],EDFT[6],EDFT[7],SXDFT[0],SXDFT[1],SXDFT[2],SXDFT[3],SXDFT[4],SXDFT[5],SXDFT[6],SXDFT[7],SYDFT[0],SYDFT[1],SYDFT[2],SYDFT[3],SYDFT[4],SYDFT[5],SYDFT[6],SYDFT[7],SZDFT[0],SZDFT[1],SZDFT[2],SZDFT[3],SZDFT[4],SZDFT[5],SZDFT[6],SZDFT[7])
	if kx[i] <= 0:
		print ( (sqrt(kx[i]**2+ky[i]**2)*1e-10).real.real,EW[0],EW[1],EW[2],EW[3],EW[4],EW[5],EW[6],EW[7],EW[8],EW[9],EW[10],EW[11],EW[12],EW[13],EW[14],EW[15],EW[16],EW[17],EW[18],EW[19],EW[20],EW[21],EW[22],EW[23],EW[24],EW[25],EW[26],EW[27],EW[28],EW[29],EW[30],EW[31],EDFT[0],EDFT[1],EDFT[2],EDFT[3],EDFT[4],EDFT[5],EDFT[6],EDFT[7],SXDFT[0],SXDFT[1],SXDFT[2],SXDFT[3],SXDFT[4],SXDFT[5],SXDFT[6],SXDFT[7],SYDFT[0],SYDFT[1],SYDFT[2],SYDFT[3],SYDFT[4],SYDFT[5],SYDFT[6],SYDFT[7],SZDFT[0],SZDFT[1],SZDFT[2],SZDFT[3],SZDFT[4],SZDFT[5],SZDFT[6],SZDFT[7])






