#!/usr/bin/python
import numpy as np
import cmath
from cmath import*
import math
from sympy.physics.quantum.dagger import Dagger
from lmfit import Model
import time
#import matplotlib.pyplot as plt

#Bilayer low energy hamiltonian from 
#S. KONSCHUH, M. GMITRA, D. KOCHAN, AND J. FABIANPHYSICAL REVIEW B85, 115423 (2012)



a = 9.83999995859962e-10
aGr = 2.46e-10
tau = 1
hbar = 6.582119e-16
pi = math.pi
Ry = 13.605693
EF = 0.0
ED = 0.00222022
gamma0 = 2.55517409
gamma1 = 0.41167674
gamma3 = -0.31096519
gamma4 = -0.17848374
V = -0.00696512
delta = 0.01337065
LIA1 = 0.00112134
LIB1 = -0.00119197
LIA2 = 12e-6
LIB2 = 12e-6
L0 = 0
LBR1 = 0
LBR2 = 0
L1 = 0.0
L3 = 0.0
L4 =0
DL4 = 0.0
# In order to add exchange coupling to one layer, we introduce on the diagonal terms for A1 and B1
BexA1 = 0.0
BexB1 = 0.0
BexA2 = 0.0
BexB2 = 0.0


#DFT daten#
data = np.loadtxt('./bandsK/DFTbandsK.dat')
kx = []
ky = []

k1 = (data[:,0]-1./3.).real 	#shift by (1/3, 1/3) due to the K point
k2 = (data[:,1]-1./3.).real
kmax=len(k1)
kcutl = 40
kcutr = 40

for i in range(kcutl,kmax-kcutr):
        kx.append(((4*pi/(sqrt(3)*a))*(k1[i]*sqrt(3.)/2.)).real)
        ky.append(((4*pi/(sqrt(3)*a))*(k1[i]*(1./2.)+k2[i])).real)

Energy = []

for i in range(kcutl,kmax-kcutr):
	B = data[i][2:10]-EF
	B.sort()
	#Energy = np.append(Energy,B)
	#Energy = np.append(Energy,B[1]-B[0])
	#Energy = np.append(Energy,B[3]-B[2])
	#Energy = np.append(Energy,B[5]-B[4])
	#Energy = np.append(Energy,B[7]-B[6])
	Energy = np.append(Energy,(data[i][10:18]))    #spinx
	Energy = np.append(Energy,(data[i][18:26]))    #spiny  
	Energy = np.append(Energy,(data[i][26:34]))    #spinz

	

# basis is: A1up, A1dn, B1up, B1dn, A2up, A2dn, B2up, B2dn
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def Hamiltonian(kx,ky,tau,ED,gamma0,gamma1,gamma3,gamma4,V,delta,LIA1,LIA2,LIB1,LIB2,L0,LBR1,LBR2,L1,L3,L4,DL4,BexA1,BexB1,BexA2,BexB2):

	#FK = exp(  complex(0.0,a*ky/  sqrt(3.0)  )  )* (  1+2*exp(  complex(  0.0,-sqrt(3.0)*a*ky/2.0  )  )*cos(a*kx/2.0)  )	#structure function, non linearized, momentum must be at K, do not shift
	#CFK = np.conjugate(FK)

	FK = (1./2.)*sqrt(3.)*(aGr)*complex(-tau*kx,ky)		#linearized version of structure function, expanded around K point, so shift momenta by (1/3,1/3), which is the K point
	CFK = np.conjugate(FK)
	
	h11 = delta+tau*LIA1+V-BexA1
	h12 = 0.0
	h13 = FK*gamma0
	h14 = (1./2.)*complex(0.0,(L0+2*LBR1)*(1-tau))
	h15 = gamma4*CFK
	h16 = (1./2.)*complex(0.0,(DL4+L4)*(1+tau))
	h17 = gamma1+L1*tau
	h18 = 0.0

	h21 = 0.0
	h22 = delta-LIA1*tau+V+BexA1
	h23 = (1./2.)*complex(0.0,(L0+2*LBR1)*(1+tau))
	h24 = FK*gamma0
	h25 = (1./2.)*complex(0.0,(DL4+L4)*(1-tau))
	h26 = gamma4*CFK
	h27 = 0.0
	h28 = -gamma1-L1*tau

	h31 = CFK*gamma0
	h32 = -(1./2.)*complex(0.0,(L0+2*LBR1)*(1+tau))
	h33 = -LIB1*tau+V-BexB1
	h34 = 0.0
	h35 = FK*gamma3
	h36 = (1./2.)*complex(0.0,L3*(1-tau))
	h37 = CFK*gamma4
	h38 = -(1./2.)*complex(0.0,(-DL4+L4)*(1+tau))

	h41 = -(1./2.)*complex(0.0,(L0+2*LBR1)*(1-tau))
	h42 = CFK*gamma0
	h43 = 0.0
	h44 = LIB1*tau+V+BexB1
	h45 = (1./2.)*complex(0.0,L3*(1+tau))
	h46 = FK*gamma3
	h47 = -(1./2.)*complex(0.0,(-DL4+L4)*(1-tau))
	h48 = CFK*gamma4

	h51 = FK*gamma4
	h52 =  -(1./2.)*complex(0.0,(DL4+L4)*(1-tau))
	h53 = CFK*gamma3
	h54 = -(1./2.)*complex(0.0,L3*(1+tau))
	h55 = LIA2*tau-V-BexA2
	h56 = 0.0
	h57 = FK*gamma0
	h58 = -(1./2.)*complex(0.0,(L0-2*LBR2)*(1-tau))

	h61 = -(1./2.)*complex(0.0,(DL4+L4)*(1+tau))
	h62 = FK*gamma4
	h63 = -(1./2.)*complex(0.0,L3*(1-tau))
	h64 = CFK*gamma3
	h65 = 0.0
	h66 = -LIA2*tau-V+BexA2
	h67 = -(1./2.)*complex(0.0,(L0-2*LBR2)*(1+tau))
	h68 = FK*gamma0

	h71 = gamma1+L1*tau
	h72 = 0.0
	h73 = FK*gamma4
	h74 = (1./2.)*complex(0.0,(-DL4+L4)*(1-tau))
	h75 = CFK*gamma0
	h76 = (1./2.)*complex(0.0,(L0-2*LBR2)*(1+tau))
	h77 = delta-LIB2*tau-V-BexB2
	h78 = 0.0

	h81 = 0.0
	h82 = gamma1-L1*tau
	h83 = (1./2.)*complex(0.0,(-DL4+L4)*(1+tau))
	h84 = FK*gamma4
	h85 = (1./2.)*complex(0.0,(L0-2*LBR2)*(1-tau))
	h86 = CFK*gamma0
	h87 = 0.0
	h88 = delta+LIB2*tau-V+BexB2
	

	H = np.array([[h11,h12,h13,h14,h15,h16,h17,h18],
		      [h21,h22,h23,h24,h25,h26,h27,h28],
		      [h31,h32,h33,h34,h35,h36,h37,h38],
		      [h41,h42,h43,h44,h45,h46,h47,h48],
		      [h51,h52,h53,h54,h55,h56,h57,h58], 
		      [h61,h62,h63,h64,h65,h66,h67,h68],
		      [h71,h72,h73,h74,h75,h76,h77,h78],
		      [h81,h82,h83,h84,h85,h86,h87,h88]])

	Sx = np.array([ [0,1,0,0,0,0,0,0],
			[1,0,0,0,0,0,0,0],
			[0,0,0,1,0,0,0,0],
			[0,0,1,0,0,0,0,0],
			[0,0,0,0,0,1,0,0],
			[0,0,0,0,1,0,0,0],
			[0,0,0,0,0,0,0,1],
			[0,0,0,0,0,0,1,0]])

	Sy = np.array([ [0,complex(0,-1),0,0,0,0,0,0],
			[complex(0,1),0,0,0,0,0,0,0],
			[0,0,0,complex(0,-1),0,0,0,0],
			[0,0,complex(0,1),0,0,0,0,0],
			[0,0,0,0,0,complex(0,-1),0,0],
			[0,0,0,0,complex(0,1),0,0,0],
			[0,0,0,0,0,0,0,complex(0,-1)],
			[0,0,0,0,0,0,complex(0,1),0]])

	Sz = np.array([ [1,0,0,0,0,0,0,0],
			[0,-1,0,0,0,0,0,0],
			[0,0,1,0,0,0,0,0],
			[0,0,0,-1,0,0,0,0],
			[0,0,0,0,1,0,0,0],
			[0,0,0,0,0,-1,0,0],
			[0,0,0,0,0,0,1,0],
			[0,0,0,0,0,0,0,-1]])

	evals, evecs = np.linalg.eigh(H)

	ew=[[],[],[],[],[],[],[],[]]
	spinx=[[],[],[],[],[],[],[],[]]
	spiny=[[],[],[],[],[],[],[],[]]
	spinz=[[],[],[],[],[],[],[],[]]



	for i in range(0,8):
		ew[i] = evals[i]+ED
		EV = evecs[:,i]
		DEV = Dagger(evecs[:,i])
		spinx[i]=1./2.*(DEV.dot(Sx.dot(EV))).real
		spiny[i]=1./2.*(DEV.dot(Sy.dot(EV))).real	
		spinz[i]=1./2.*(DEV.dot(Sz.dot(EV))).real	
	
	
	return spinx[0],spinx[1],spinx[2],spinx[3],spinx[4],spinx[5],spinx[6],spinx[7],spiny[0],spiny[1],spiny[2],spiny[3],spiny[4],spiny[5],spiny[6],spiny[7],spinz[0],spinz[1],spinz[2],spinz[3],spinz[4],spinz[5],spinz[6],spinz[7]
	
        #ew[0],ew[1],ew[2],ew[3],ew[4],ew[5],ew[6],ew[7],ew[8],ew[9],ew[10],ew[11]
        #ew[1]-ew[0],ew[3]-ew[2],ew[5]-ew[4],ew[7]-ew[6],ew[9]-ew[8],ew[11]-ew[10]
        #spinx[0],spinx[1],spinx[2],spinx[3],spinx[4],spinx[5],spinx[6],spinx[7],spinx[8],spinx[9],spinx[10],spinx[11]
        #spiny[0],spiny[1],spiny[2],spiny[3],spiny[4],spiny[5],spiny[6],spiny[7],spiny[8],spiny[9],spiny[10],spiny[11]
        #spinz[0],spinz[1],spinz[2],spinz[3],spinz[4],spinz[5],spinz[6],spinz[7],spinz[8],spinz[9],spinz[10],spinz[11]
	
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


def Hsum(kx,LBR1,LBR2): 

	E = []
	for i in range(0,len(kx)):
			EW = Hamiltonian(kx[i],ky[i],tau,ED,gamma0,gamma1,gamma3,gamma4,V,delta,LIA1,LIA2,LIB1,LIB2,L0,LBR1,LBR2,L1,L3,L4,DL4,BexA1,BexB1,BexA2,BexB2)
			E = np.append(E,EW)
	return E




gmod = Model(Hsum) # Lade FUNKTION

result = gmod.fit(Energy,kx=kx,LBR1=-0.5e-3,LBR2 = -0.1e-3, method = 'leastsq')
	
	

print(result.fit_report())






